@@ -596,7 +596,8 @@ class TerminalMenu:
         status_bar: Optional[Union[str, Iterable[str], Callable[[str], str]]] = None,
         status_bar_below_preview: bool = DEFAULT_STATUS_BAR_BELOW_PREVIEW,
         status_bar_style: Optional[Iterable[str]] = DEFAULT_STATUS_BAR_STYLE,
-        title: Optional[Union[str, Iterable[str]]] = None
+        title: Optional[Union[str, Iterable[str]]] = None,
+        explode_on_interrupt: bool = False
     ):
         def extract_shortcuts_menu_entries_and_preview_arguments(
             entries: Iterable[str],
@@ -618,7 +619,7 @@ class TerminalMenu:
                 else:
                     unit_separated_entry = escaped_separator_pattern.sub("|", separator_pattern.sub("\\1\x1F", entry))
                     match_obj = menu_entry_pattern.match(unit_separated_entry)
-                    # this is none in case the entry was an emtpy string which
+                    # this is none in case the entry was an empty string which
                     # will be interpreted as a separator
                     assert match_obj is not None
                     shortcut_key = match_obj.group(1)
@@ -718,6 +719,7 @@ class TerminalMenu:
         self._search_case_sensitive = search_case_sensitive
         self._search_highlight_style = tuple(search_highlight_style) if search_highlight_style is not None else ()
         self._search_key = search_key
+        self._explode_on_interrupt = explode_on_interrupt
         self._shortcut_brackets_highlight_style = (
             tuple(shortcut_brackets_highlight_style) if shortcut_brackets_highlight_style is not None else ()
         )
@@ -1538,7 +1540,9 @@ class TerminalMenu:
                         # Only append `next_key` if it is a printable character and the first character is not the
                         # `search_start` key
                         self._search.search_text += next_key
-        except KeyboardInterrupt:
+        except KeyboardInterrupt as e:
+            if self._explode_on_interrupt:
+                raise e
             menu_was_interrupted = True
         finally:
             reset_signal_handling()
@@ -1841,6 +1845,12 @@ def get_argumentparser() -> argparse.ArgumentParser:
         ),
     )
     parser.add_argument("-t", "--title", action="store", dest="title", help="menu title")
+    parser.add_argument(
+        "--explode-on-interrupt",
+        action="store_true",
+        dest="explode_on_interrupt",
+        help="Instead of quitting the menu, this will raise the KeyboardInterrupt Exception",
+    )
     parser.add_argument(
         "-V", "--version", action="store_true", dest="print_version", help="print the version number and exit"
     )
@@ -1971,6 +1981,7 @@ def main() -> None:
             status_bar_below_preview=args.status_bar_below_preview,
             status_bar_style=args.status_bar_style,
             title=args.title,
+            explode_on_interrupt=args.explode_on_interrupt,
         )
     except (InvalidParameterCombinationError, InvalidStyleError, UnknownMenuEntryError) as e:
         print(str(e), file=sys.stderr)
