@@ -1,5 +1,5 @@
 from .save_conf import save_config
-from .manage_users_conf import ask_for_superuser_account, ask_for_additional_users
+from .manage_users_conf import ask_for_additional_users
 from .backwards_compatible_conf import generic_select, generic_multi_select
 from .locale_conf import select_locale_lang, select_locale_enc
 from .system_conf import select_kernel, select_harddrives, select_driver, ask_for_bootloader, ask_for_swap
