@@ -7,9 +7,6 @@ is_top_level_profile = False
 # New way of defining packages for a profile, which is iterable and can be used out side
 # of the profile to get a list of "what packages will be installed".
 __packages__ = [
-	"nemo",
-	"gpicview",
-	"maim",
 	"alacritty",
 ]
 
@@ -51,8 +48,4 @@ if __name__ == 'awesome':
 	with open(f"{archinstall.storage['installation_session'].target}/etc/xdg/awesome/rc.lua", 'w') as fh:
 		fh.write(awesome_lua)
 
-	# TODO: Configure the right-click-menu to contain the above packages that were installed. (as a user config)
-
-	# Remove some interfering nemo settings
-	archinstall.storage['installation_session'].arch_chroot("gsettings set org.nemo.desktop show-desktop-icons false")
-	archinstall.storage['installation_session'].arch_chroot("xdg-mime default nemo.desktop inode/directory application/x-gnome-saved-search")
+	# TODO: Configure the right-click-menu to contain the above packages that were installed. (as a user config)
\ No newline at end of file
